// Load the modules that this module needs
var Crypto = require('crypto');

/**
 * DeskApp encryption module. Can be used to encrypt and decrypt
 * strings.
 * If this module is updated, the stored passwords also need to be updated,
 * so we need to be able to decrypt using the old versions method. For
 * that reason the methods are kept with their version.
 */
var DAEncryption = (function() {

	"use strict";

	// The version of this module. If the encryption method is updated,
	// the version should also be updated, because the version will be
	// stored with encrypted passwords, so we can replace old encrypted
	// passwords with new encrypted passwords
	var _version = 'v0.1';

	var _DAEncryption = {};

	// The encryption methods of version v0.1 of this module.
	// If the methods are changed they should be added as a new version
	// and this version should be kept intact because it is used to
	// decrypt stored passwords before they are encrypted with the
	// new method.
	_DAEncryption['v0.1'] = (function() {

		var _key = global.DA.readAppData('dakey');
		if (!_key) {
			// Create a key and store it
			_key = Crypto.randomBytes(256).toString('hex');
			console.log('key created: ', _key);
			global.DA.saveAppData('dakey', _key);
		}

		// The algorithm (supported by OpenSSL) that will be used for the
		// encryption and decryption
		var _algorithm = 'aes256';

		var _DAE = function() {};

		/**
		 * Encrypts a string
		 * @param {String} str The string that needs to be encrypted
		 * @return {String} The encrypted string
		 */
		_DAE.prototype.encrypt = function(str) {
			try {
				var cipher = Crypto.createCipher(_algorithm, _key);
				return cipher.update(str, 'utf8', 'hex') + cipher.final('hex');
			} catch (e) {
				console.log('Encryption failed: ', e);
			}

			return '';
		};

		/**
		 * Decrypts a string
		 * @param {String} str The encrypted string that needs to decrypted
		 * @return {String} The decrypted string
		 */
		_DAE.prototype.decrypt = function(str) {
			try {
				var decipher = Crypto.createDecipher(_algorithm, _key);
				return decipher.update(str, 'hex', 'utf8') + decipher.final('utf8');
			} catch (e) {
				console.log('Decryption failed: ', e);
			}

			return '';
		};

		return new _DAE();
	})();

	// Let the encrypt and decrypt methods point to the current version
	_DAEncryption.encrypt = _DAEncryption[_version].encrypt;
	_DAEncryption.decrypt = _DAEncryption[_version].decrypt;

	// Make the version publicly available
	_DAEncryption.getVersion = function() {
		return _version;
	};

	return _DAEncryption;
})();
